/**
 * A command line client and web services testing client for    
 * JAva Bioinformatics Analysis Web Services. 
 * 
 * @author Peter Troshin 
 * @version 1.0	April 2010
 * 
 */
package compbio.ws.client;

