/**
 * A data model for multiple sequence alignment web services and utility methods
 * that work on the objects of this model.  
 * Classes in this package have no dependencies to other sources in the project. 
 * They form a base layer of JAva Bioinformatics Analysis Web Services. 
 * 
 * @author Petr Troshin 
 * @version 1.0	January 2010
 * 
 * 
 */
package compbio.data.sequence;

