/**
 * A meta-data model for multiple sequence alignment web services 
 * Classes in this package have no dependencies to other sources in the project. 
 * They form a base layer of JAva Bioinformatics Analysis Web Services (JABAWS)
 * 
 * @author Peter Troshin 
 * 
 * @version 1.0 January 2010
 */
package compbio.metadata;

