
package compbio.data.msa.jaxws;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;


/**
 * This class was generated by the JAX-WS RI.
 * JAX-WS RI 2.2.3-b01-
 * Generated source version: 2.2.3
 * 
 */
@XmlRootElement(name = "LimitExceededException", namespace = "http://msa.data.compbio/01/12/2010/")
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "LimitExceededException", namespace = "http://msa.data.compbio/01/12/2010/", propOrder = {
    "actualNumberofSequences",
    "message",
    "numberOfSequencesAllowed",
    "sequenceLenghtActual",
    "sequenceLenghtAllowed"
})
public class LimitExceededExceptionBean {

    private int actualNumberofSequences;
    private String message;
    private int numberOfSequencesAllowed;
    private int sequenceLenghtActual;
    private int sequenceLenghtAllowed;

    /**
     * 
     * @return
     *     returns int
     */
    public int getActualNumberofSequences() {
        return this.actualNumberofSequences;
    }

    /**
     * 
     * @param actualNumberofSequences
     *     the value for the actualNumberofSequences property
     */
    public void setActualNumberofSequences(int actualNumberofSequences) {
        this.actualNumberofSequences = actualNumberofSequences;
    }

    /**
     * 
     * @return
     *     returns String
     */
    public String getMessage() {
        return this.message;
    }

    /**
     * 
     * @param message
     *     the value for the message property
     */
    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * 
     * @return
     *     returns int
     */
    public int getNumberOfSequencesAllowed() {
        return this.numberOfSequencesAllowed;
    }

    /**
     * 
     * @param numberOfSequencesAllowed
     *     the value for the numberOfSequencesAllowed property
     */
    public void setNumberOfSequencesAllowed(int numberOfSequencesAllowed) {
        this.numberOfSequencesAllowed = numberOfSequencesAllowed;
    }

    /**
     * 
     * @return
     *     returns int
     */
    public int getSequenceLenghtActual() {
        return this.sequenceLenghtActual;
    }

    /**
     * 
     * @param sequenceLenghtActual
     *     the value for the sequenceLenghtActual property
     */
    public void setSequenceLenghtActual(int sequenceLenghtActual) {
        this.sequenceLenghtActual = sequenceLenghtActual;
    }

    /**
     * 
     * @return
     *     returns int
     */
    public int getSequenceLenghtAllowed() {
        return this.sequenceLenghtAllowed;
    }

    /**
     * 
     * @param sequenceLenghtAllowed
     *     the value for the sequenceLenghtAllowed property
     */
    public void setSequenceLenghtAllowed(int sequenceLenghtAllowed) {
        this.sequenceLenghtAllowed = sequenceLenghtAllowed;
    }

}
